"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createPages = void 0;
const fs_1 = __importDefault(require("fs"));
const utils_1 = require("../utils");
const create_query_executor_1 = require("./create-query-executor");
const drupal_feeds_1 = require("./drupal-feeds");
const createPages = (args, options) => __awaiter(void 0, void 0, void 0, function* () {
    if (!(0, utils_1.validOptions)(options)) {
        return;
    }
    const executor = (0, create_query_executor_1.createQueryExecutor)(options);
    const feeds = yield (0, drupal_feeds_1.drupalFeeds)(executor);
    for (const feed of feeds) {
        if (!feed.pathFieldName) {
            continue;
        }
        const { data, errors } = yield args.graphql(`
          query {
            list: allDrupal${feed.typeName} {
              nodes {
                typeName: __typename
                id
                remoteId
                path: ${feed.pathFieldName}
                ${feed.templateFieldName
            ? `template: ${feed.templateFieldName}`
            : ''}
                ${feed.translatable
            ? `localizations: translations {
                         path: ${feed.pathFieldName}
                         locale: langcode
                       }`
            : ''}
              }
            }
          }
        `);
        if (!data) {
            console.error('errors', errors);
            throw new Error(`Cannot fetch create-page fields for "${feed.typeName}" type from Gatsby.`);
        }
        data.list.nodes.forEach((node) => {
            var _a;
            if (!node.path) {
                return;
            }
            const { typeName, id, remoteId, path } = node;
            const [, locale] = remoteId.split(':');
            const template = node.template ||
                // Generate the template name from the type.
                // E.g. "BlogPost" => "blog-post".
                feed.typeName.replace(/([a-z0-9])([A-Z])/g, '$1-$2').toLowerCase();
            const context = {
                typeName,
                id,
                remoteId,
                locale,
                localizations: (_a = node.localizations) === null || _a === void 0 ? void 0 : _a.filter((it) => !!it.path),
            };
            const templatesPath = `${args.store.getState().program.directory}/src/templates`;
            const templatePath = `${templatesPath}/${template}.tsx`;
            if (fs_1.default.existsSync(templatePath)) {
                args.actions.createPage({
                    path,
                    component: require.resolve(templatePath),
                    context,
                });
            }
            else {
                args.reporter.warn(`Could not find "${template}.tsx" template in "${templatesPath}" directory. The stub template will be used for node "${node.remoteId}" of type "${feed.typeName}" on "${path}" path.`);
                args.actions.createPage({
                    path,
                    component: require.resolve(`${__dirname}/../templates/stub.js`),
                    context: Object.assign(Object.assign({}, context), { expectedTemplatePath: templatePath }),
                });
            }
        });
    }
});
exports.createPages = createPages;
