"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createNetworkQueryExecutor = exports.createQueryExecutor = void 0;
const crypto_1 = require("crypto");
const fetch_retry_ts_1 = __importDefault(require("fetch-retry-ts"));
const fs_1 = require("fs");
const gatsby_graphql_source_toolkit_1 = require("gatsby-graphql-source-toolkit");
const node_fetch_1 = __importDefault(require("node-fetch"));
const util_1 = require("util");
const fetchWithRetries = (0, fetch_retry_ts_1.default)(node_fetch_1.default);
const createQueryExecutor = (options) => {
    const url = `${new URL(options.drupal_url).origin}${options.graphql_path}`;
    const executor = createNetworkQueryExecutor(url, {
        headers: Object.assign(Object.assign(Object.assign({}, options === null || options === void 0 ? void 0 : options.headers), ((options === null || options === void 0 ? void 0 : options.auth_user) && (options === null || options === void 0 ? void 0 : options.auth_pass)
            ? {
                Authorization: `Basic ${Buffer.from(`${options.auth_user}:${options.auth_pass}`).toString('base64')}`,
            }
            : {})), ((options === null || options === void 0 ? void 0 : options.auth_key) ? { 'api-key': options.auth_key } : {})),
        timeout: 60000,
    });
    return (0, gatsby_graphql_source_toolkit_1.wrapQueryExecutorWithQueue)(executor, {
        concurrency: options.query_concurrency || 10,
    });
};
exports.createQueryExecutor = createQueryExecutor;
// Copy of the default network query executor from gatsby-graphql-source-toolkit
// package with improved error handling.
// https://github.com/gatsbyjs/gatsby-graphql-toolkit/blob/7ba8b24d8524254dc6c2853d6d014070befd9aea/src/config/query-executor.ts#L6-L39
function createNetworkQueryExecutor(uri, fetchOptions = {}) {
    return function execute(args) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const { query, variables, operationName } = args;
            let response;
            try {
                response = yield fetchWithRetries(uri, Object.assign(Object.assign({ method: 'POST', body: JSON.stringify({ query, variables, operationName }) }, fetchOptions), { headers: Object.assign({ 'Content-Type': 'application/json' }, fetchOptions.headers) }));
            }
            catch (e) {
                console.error(`Query ${operationName} failed: ${e}\n` +
                    `Query variables: ${(0, util_1.inspect)(variables)}\n` +
                    `Full query: ${logQuery(query)}\n`);
                throw e;
            }
            if (!response.ok) {
                console.warn(`Query ${operationName} returned status ${response.status}.\n` +
                    `Query variables: ${(0, util_1.inspect)(variables)}\n` +
                    `Full query: ${logQuery(query)}\n`);
            }
            const result = yield response.json();
            if (result.data && ((_a = result.errors) === null || _a === void 0 ? void 0 : _a.length)) {
                console.warn(`Query ${operationName} returned warnings:\n` +
                    `${(0, util_1.inspect)(result.errors)}\n` +
                    `Query variables: ${(0, util_1.inspect)(variables)}\n` +
                    `Full query: ${logQuery(query)}\n`);
            }
            return result;
        });
    };
}
exports.createNetworkQueryExecutor = createNetworkQueryExecutor;
const logQuery = (query) => {
    const hash = (0, crypto_1.createHash)('md5').update(query).digest('hex');
    const filename = `/tmp/gatsby-source-silverback--failed-query--${hash}.txt`;
    try {
        (0, fs_1.writeFileSync)(filename, query);
        return filename;
    }
    catch (e) {
        return '[could not store query as file]';
    }
};
