"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchNodeChanges = void 0;
const gatsby_graphql_source_toolkit_1 = require("gatsby-graphql-source-toolkit");
const fetchNodeChanges = (lastBuildId, currentBuildId, reporter, context, execute) => __awaiter(void 0, void 0, void 0, function* () {
    var e_1, _a;
    const result = [];
    const changedContent = (yield execute({
        operationName: 'ContentChanges',
        variables: { lastBuildId, currentBuildId },
        query: `
      query ContentChanges($lastBuildId: Int!, $currentBuildId: Int!) {
        drupalFeedInfo {
          typeName
          changes(lastBuild: $lastBuildId, currentBuild: $currentBuildId)
        }
      }
    `,
    }));
    if (!changedContent.data) {
        reporter.warn(`Cannot fetch content updates.\n ${JSON.stringify(changedContent)}`);
        throw new Error('Cannot fetch content updates.');
    }
    for (let i = 0; i < changedContent.data.drupalFeedInfo.length; i++) {
        const feed = changedContent.data.drupalFeedInfo[i];
        const nodes = (0, gatsby_graphql_source_toolkit_1.fetchNodesById)(context, feed.typeName, feed.changes.map((id) => ({ id })));
        let index = 0;
        try {
            for (var nodes_1 = (e_1 = void 0, __asyncValues(nodes)), nodes_1_1; nodes_1_1 = yield nodes_1.next(), !nodes_1_1.done;) {
                const node = nodes_1_1.value;
                result.push({
                    eventName: node ? 'UPDATE' : 'DELETE',
                    remoteTypeName: feed.typeName,
                    remoteId: {
                        __typename: feed.typeName,
                        id: feed.changes[index],
                    },
                });
                index++;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (nodes_1_1 && !nodes_1_1.done && (_a = nodes_1.return)) yield _a.call(nodes_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    }
    // Report results.
    const updated = result.filter((it) => it.eventName === 'UPDATE');
    reporter.info(`ℹ️ sourceNodes will (re)fetch ${updated.length} nodes`);
    const deleted = result.filter((it) => it.eventName === 'DELETE');
    reporter.info(`ℹ️ sourceNodes will delete ${deleted.length} nodes`);
    return result;
});
exports.fetchNodeChanges = fetchNodeChanges;
